==> unixODBC-1.8.8.tar.gz: unixODBC-1.8.8/sqi/sqiPrimaryKeys.c <==
#include "sqi.h"

int sqiPrimaryKeys( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
	HSQITABLE	hTableRoot;
	int			nCol;

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( pszTableName );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pszTableName );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * BUILD RESULTS
	 ************************/
	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiPrimaryKeys" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiPrimaryKeys", "TABLE_CAT" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiPrimaryKeys", "TABLE_SCHEM" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiPrimaryKeys", "TABLE_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiPrimaryKeys", "COLUMN_NAME" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiPrimaryKeys", "KEY_SEQ" ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiPrimaryKeys", "PK_NAME" ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	nCol = 0;
	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pColumn	= (HSQICOLUMN)lstGet( hColHdrs );
		if ( pColumn->bPimaryKey )
		{
			nCol++;
			pData		= malloc( sizeof(char*) * 7 );
			pData[0]	= (char *)strdup( "" );
			pData[1]	= (char *)strdup( "" );
			pData[2]	= (char *)strdup( pszTableName );
			pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );
			sprintf( szBuffer, "%d", nCol );
			pData[4]	= (char *)strdup( szBuffer );
			pData[5]	= (char *)strdup( "PKEY" );
	
			pData[6]	= NULL; /* data terminator... see _sqiFreeDataRow */
	
			lstAppend( hStmt->hTableResult->hRows, pData );
		}

		lstNext( hColHdrs );
	}
	lstClose( hColHdrs );

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}

==> unixODBC-1.8.13.tar.gz: unixODBC-1.8.13/sqi/sqiPrimaryKeys.c <==
#include "sqi.h"

int sqiPrimaryKeys( HSQISTMT hStmt, char *pszTableName )
{
	HSQI		hSQI;
	HLST		hColHdrs;
	char		szBuffer[101];
	char		**pData;
	HSQICOLUMN	pColumn;
	HSQITABLE	hTableRoot;
	int			nCol;

	if ( !hStmt )
		return false;
	
	if ( !pszTableName )
		return false;

	if ( hStmt->hTableResult )
		sqiFreeResult( hStmt );

	hSQI	= (HSQI)hStmt->hSQI;

	/************************
	 * GET REF TO ROOT TABLE FROM TABLE CACHE
	 ************************/
	hTableRoot = _sqiGetTableHandle( hSQI, pszTableName );
	if ( !hTableRoot )
	{
		sprintf( hStmt->szMsg, "Could not find table (%s)", pszTableName );
		logPushMsg( hStmt->hLog, __FILE__, __FILE__, __LINE__, LOG_CRITICAL, SQI_ERROR, hStmt->szMsg );
		return false;
	}

	/************************
	 * BUILD RESULTS
	 ************************/
	hStmt->hTableResult 				= malloc( sizeof(SQITABLE) );

	hStmt->hTableResult->pszTableName	= (char*)strdup( "sqiPrimaryKeys" );
	hStmt->hTableResult->pszFileName	= NULL;
	hStmt->hTableResult->nRefs			= 0;
	hStmt->hTableResult->hBase			= NULL;

	/* table header */
	hStmt->hTableResult->hColHdrs	= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hColHdrs, _sqiFreeColumnHeader );

	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 0, "system", "sqiPrimaryKeys", "TABLE_CAT", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 1, "system", "sqiPrimaryKeys", "TABLE_SCHEM", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 2, "system", "sqiPrimaryKeys", "TABLE_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 3, "system", "sqiPrimaryKeys", "COLUMN_NAME", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 4, "system", "sqiPrimaryKeys", "KEY_SEQ", SQL_VARCHAR, 50 ) );
	lstAppend( hStmt->hTableResult->hColHdrs, _sqiNewColumnHeader( 5, "system", "sqiPrimaryKeys", "PK_NAME", SQL_VARCHAR, 50 ) );

	/* table rows (based upon root table col hdrs) */
	hStmt->hTableResult->hRows		= lstOpen();
	lstSetFreeFunc( hStmt->hTableResult->hRows, _sqiFreeDataRow );

	nCol = 0;
	lstFirst( hColHdrs );
	while ( !lstEOL( hColHdrs ) )
	{
		pColumn	= (HSQICOLUMN)lstGet( hColHdrs );
		if ( pColumn->bPimaryKey )
		{
			nCol++;
			pData		= malloc( sizeof(char*) * 7 );
			pData[0]	= (char *)strdup( "" );
			pData[1]	= (char *)strdup( "" );
			pData[2]	= (char *)strdup( pszTableName );
			pData[3]	= (char *)strdup( pColumn->pszSQL_DESC_BASE_COLUMN_NAME );
			sprintf( szBuffer, "%d", nCol );
			pData[4]	= (char *)strdup( szBuffer );
			pData[5]	= (char *)strdup( "PKEY" );
	
			pData[6]	= NULL; /* data terminator... see _sqiFreeDataRow */
	
			lstAppend( hStmt->hTableResult->hRows, pData );
		}

		lstNext( hColHdrs );
	}
	lstClose( hColHdrs );

	hStmt->nRowsAffected	= hStmt->hTableResult->hRows->nItems;

	/* we should be EOL for call to fetch */
	lstLast( hStmt->hTableResult->hRows );
	lstNext( hStmt->hTableResult->hRows );

	return true;
}
